@@ -8,10 +8,10 @@ from urllib import urlencode
 import requests
 from oauth_hook import OAuthHook
 
-from query import Crosswalk, Resolve, Table
+from query import Crosswalk, Resolve, Table, Contribute, Facets, Flag
 
 API_V3_HOST = "http://api.v3.factual.com"
-DRIVER_VERSION_TAG = "factual-python-driver-1.0.0"
+DRIVER_VERSION_TAG = "factual-python-driver-1.1.0"
 
 class Factual(object):
     def __init__(self, key, secret):
@@ -26,11 +26,20 @@ class Factual(object):
         return Crosswalk(self.api)
 
     def resolve(self, values):
-        return Resolve(self.api, values)
+        return Resolve(self.api, {'values': values})
 
     def raw_read(self, path, raw_params):
         return self.api.raw_read(path, raw_params)
 
+    def facets(self, table):
+        return Facets(self.api, 't/' + table + '/facets')
+
+    def contribute(self, table, factual_id=None, values={}):
+        return Contribute(self.api, table, factual_id, {'values': values})
+
+    def flag(self, table, factual_id):
+        return Flag(self.api, table, factual_id)
+
     def _generate_token(self, key, secret):
         access_token = OAuthHook(consumer_key=key, consumer_secret=secret, header_auth=True)
         return access_token
@@ -40,9 +49,13 @@ class API(object):
     def __init__(self, access_token):
         self.client = requests.session(hooks={'pre_request': access_token})
 
-    def execute(self, query):
+    def get(self, query):
         response = self._handle_request(query.path, query.params)
         return response
+
+    def post(self, query):
+        response = self._make_post_request(query.path, query.params)
+        return response
         
     def schema(self, query):
         response = self._handle_request(query.path + '/schema', query.params)
@@ -50,7 +63,7 @@ class API(object):
 
     def raw_read(self, path, raw_params):
         url = self._build_base_url(path) + raw_params
-        return self._make_request(url)
+        return self._make_request(url).text
 
     def build_url(self, path, params):
         url = self._build_base_url(path) + self._make_query_string(params)
@@ -62,15 +75,24 @@ class API(object):
     def _handle_request(self, path, params):
         url = self.build_url(path, params)
         response = self._make_request(url)
-        payload = json.loads(response)
+        payload = json.loads(response.text)
         if payload['status'] != 'ok':
-            raise APIException(payload['error_type'] + ' - ' + payload['message'])
+            raise APIException(response.status_code, payload)
         return payload['response']
 
     def _make_request(self, url):
         headers = {'X-Factual-Lib': DRIVER_VERSION_TAG}
         response = self.client.get(url, headers=headers)
-        return response.text
+        return response
+
+    def _make_post_request(self, path, params):
+        url = self.build_url(path, params)
+        headers = {'X-Factual-Lib': DRIVER_VERSION_TAG}
+        response = self.client.post(url, headers=headers)
+        payload = json.loads(response.text)
+        if payload['status'] != 'ok':
+            raise APIException(response.status_code, payload)
+        return payload['response'] if 'response' in payload else payload
 
     def _make_query_string(self, params):
         string_params = []
@@ -81,4 +103,13 @@ class API(object):
 
 
 class APIException(Exception):
-    pass
+    def __init__(self, status_code, response):
+        self.status_code = status_code
+        self.response = response
+        Exception.__init__(self, response['error_type'] + ' - ' + response['message'])
+
+    def get_status_code(self):
+        return self.status_code
+
+    def get_response(self):
+        return self.response
