@@ -9,6 +9,7 @@ class FactualAPITestSuite(unittest.TestCase):
     def setUp(self):
         self.factual = Factual(KEY, SECRET)
         self.places = self.factual.table('places')
+        self.facets = self.factual.facets('global')
 
     def test_search(self):
         q = self.places.search('factual')
@@ -65,8 +66,8 @@ class FactualAPITestSuite(unittest.TestCase):
     def test_schema(self):
         schema = self.places.schema()
         self.assertEqual(21, len(schema['fields']))
-        self.assertTrue('title' in schema)
-        self.assertTrue('locality' in set(f['name'] for f in schema['fields']))
+        self.assertIn('title', schema)
+        self.assertIn('locality', set(f['name'] for f in schema['fields']))
 
     # full text search for things where locality equals 大阪市 (Osaka: test unicode)
     def test_unicode(self):
@@ -85,7 +86,7 @@ class FactualAPITestSuite(unittest.TestCase):
         self.assertIn(row['locality'], ["Santa Monica","Los Angeles","Culver City"])
 
     def test_and(self):
-        q = self.places.filters({"$and":[{"country":"US"},{"website":{"$blank":"false"}}]})
+        q = self.places.filters({"$and":[{"country":"US"},{"website":{"$blank":False}}]})
         row = q.data()[0]
         self.assertEqual('US', row['country'])
         self.assertRegexpMatches(row['website'], 'http')
@@ -98,6 +99,41 @@ class FactualAPITestSuite(unittest.TestCase):
         self.assertEqual(15, payload['response']['included_rows'])
         self.assertTrue(all(row['name'] == 'Starbucks' for row in data))
 
+    def test_facets1(self):
+        q = self.facets.search("starbucks").select("country")
+        results = q.data()['country']
+        self.assertTrue(results['us'] > 5000)
+        self.assertTrue(results['ca'] > 200)
+
+    def test_facets2(self):
+        q = self.facets.search("starbucks").select("locality,region").filters({"country":"US"})
+        locality = q.data()['locality']
+        region = q.data()['region']
+        self.assertTrue(locality['chicago'] > 50)
+        self.assertTrue(region['tx'] > 200)
+
+    def test_facets_geo(self):
+        q = self.facets.select("category").geo(circle(34.06018, -118.41835, 5000))
+        category = q.data()['category']
+        self.assertTrue(category['shopping'] > 1000)
+        self.assertTrue(category['health & medicine > physicians'] > 1000)
+
+#    def test_contribute_without_id(self):
+#        values = {'name': 'factual', 'locality': 'los angeles', 'address': '1801 Ave of the Stars'}
+#        contribute = self.factual.contribute('global', values=values).user('python_driver_tester')
+#        response = contribute.write()
+#        self.assertFalse(response['new_entity'])
+#
+#    def test_contribute_with_id(self):
+#        values = {'name': 'Factual'}
+#        contribute = self.factual.contribute('global', factual_id='03c26917-5d66-4de9-96bc-b13066173c65', values=values).user('python_driver_tester')
+#        response = contribute.write()
+#        self.assertFalse(response['new_entity'])
+#
+#    def test_flag(self):
+#        flag = self.factual.flag('global', '03c26917-5d66-4de9-96bc-b13066173c65').user('python_driver_tester').other().debug(True)
+#        response = flag.write()
+#        self.assertEqual('ok', response['status'])
 
 if __name__ == '__main__':
     unittest.main()
